/**
 * -----------------------------------------------------------------------------
 *
 * SyL - Web Application Framework
 *
 * PHP version 4 (>= 4.3.x) or 5
 *
 * Copyright (C) 2006-2007 k.watanabe
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -----------------------------------------------------------------------------
 * @package   SyL
 * @author    k.watanabe <k.watanabe@syl.jp>
 * @copyright 2006-2007 k.watanabe
 * @license   http://www.opensource.org/licenses/lgpl-license.php
 * @version   CVS: $Id:$
 * @link      http://www.syl.jp/
 * -----------------------------------------------------------------------------
 */

/**
 * ץ쥹С饹
 *
 * @package   SyL
 * @author    k.watanabe <k.watanabe@syl.jp>
 * @copyright 2006-2007 k.watanabe
 * @license   http://www.opensource.org/licenses/lgpl-license.php
 * @version   CVS: $Id:$
 * @link      http://www.syl.jp/
 */
function SyL_AjaxProgressBar(url, id, type, width, height)
{
  this.url    = url;
  this.id     = id;
  this.type   = (type) ? type : 'standard';
  this.width  = (typeof(width) == "number")  ? String(width)  : width;
  this.height = (typeof(height) == "number") ? String(height) : height;
  this.idbar  = '_syl_bar_' + id;
  this.idtd   = '_syl_td_' + id;
  this.percentage = true;
  this.created = false;

  if (!this.width) {
    if (document.getElementById(id).currentStyle) {
      this.width  = document.getElementById(id).currentStyle['width'];
    } else if (getComputedStyle) {
      this.width = getComputedStyle(document.getElementById(id)).getPropertyValue('width');
    } else {
      alert('unabled to get width');
      return;
    }
  }

  this.width = this.width.match(/^[0-9]+/);

  if (!this.height) {
    if (document.getElementById(id).currentStyle) {
      this.height  = document.getElementById(id).currentStyle['height'];
    } else if (getComputedStyle) {
      this.height = getComputedStyle(document.getElementById(id)).getPropertyValue('height');
    } else {
      alert('unabled to get height');
      return;
    }
  }

  this.height = this.height.match(/^[0-9]+/);
}

SyL_AjaxProgressBar.prototype = {

  displayPercentage: function(percentage)
  {
    this.percentage = (percentage) ? true : false;
  },

  create: function(option)
  {
    var width1;
    var width2;

    width2 = (this.percentage) ? 36 : 0;
    width1 = this.width - width2;

    var blank = 4;

    var progress = document.getElementById(this.id);
    progress.innerHTML = '';

    progress.style.width       = this.width + "px";
    progress.style.height      = "auto";
    progress.style.border      = "1px";
    progress.style.borderStyle = "solid";
    progress.style.borderColor = "#999999";
    progress.style.backgroundColor  = "#D6D3CE";

    var table = document.createElement("table");
    /*table.style.width  = "100%";*/
    table.style.margin = "2px";
    table.style.borderCollapse = "collapse";
    progress.appendChild(table);

    var tbody = document.createElement("tbody");
    table.appendChild(tbody);

    var tr = document.createElement("tr");
    tbody.appendChild(tr);

    var td = document.createElement("td");
    td.style.width       = width1 + "px";
    td.style.border      = "1px";
    td.style.borderStyle = "solid";
    td.style.borderColor = "#BBBBBB";
    td.style.backgroundColor  = "#EEEEEE";
    tr.appendChild(td);

    var bar;
    switch (this.type) {
    case 'smooth':
      bar = this.createSmooth(this.height - blank, option);
      break;
    default:
      bar = this.createStandard(this.height - blank, option);
      break;
    }
    td.appendChild(bar);

    if (width2 > 0) {
      var td = document.createElement("td");
      td.setAttribute("id", this.idtd);
      td.style.width         = width2 + "px";
      td.style.margin        = "0px";
      td.style.padding       = "0px";
      td.style.fontSize      = "12px";
      td.style.textAlign     = "center";
      td.style.verticalAlign = "middle";
      td.innerHTML = "0%";
      tr.appendChild(td);
    } else {
      this.idtd = '';
    }

    this.created = true;
  },

  createStandard: function(height, option)
  {
    var table = document.createElement("table");
    table.style.width   = "100%";
    table.style.margin  = "0px";
    table.style.padding = "0px";
    table.style.borderCollapse = "separate";

    var tbody = document.createElement("tbody");
    table.appendChild(tbody);

    var tr = document.createElement("tr");
    tbody.appendChild(tr);

    var separate = (option && (option.separate)) ? option.separate : 10;

    var idbar = '';
    for (var i=1; i<=separate; i++) {
      var tmp = this.idbar + String(i);
      var td = document.createElement("td");
      td.setAttribute("id", tmp);
      td.style.height = height + "px";
      td.style.backgroundColor = "#EEEEEE";
      tr.appendChild(td);
      idbar += (idbar == '') ? tmp : ',' + tmp;
    }
    this.idbar = idbar;

    return table;
  },

  createSmooth: function(height, option)
  {
    var div = document.createElement("div");
    div.setAttribute("id", this.idbar);
    div.style.width           = "0%";
    div.style.height          = height + "px";
    div.style.backgroundColor = "#0B2467";
    return div;
  },

  clear: function()
  {
    if (document.getElementById(this.idtd)) {
      document.getElementById(this.idtd).innerHTML = "0%";
    }
    switch (this.type) {
    case 'smooth':
      bar = this.clearSmooth();
      break;
    default:
      bar = this.clearStandard();
      break;
    }
  },

  clearStandard: function()
  {
    var idss  = this.idbar.split(",");
    var num   = idss.length;
    for (var i=0; i<num; i++) {
      document.getElementById(idss[i]).style.backgroundColor  = "#EEEEEE";
    }
  },

  clearSmooth: function()
  {
    document.getElementById(this.idbar).style.width = "0%";
  },

  start: function(form)
  {
    var self = this;
    if (!self.created) {
      self.create();
    }

    var s = SyL_AjaxLoading.process(
      form,
      self.url,
      {
        'start'   : this.startCallback,
        'loading' : this.loadingCallback,
        'loaded'  : this.completeCallback,
        'exclusive' : this.id
      },
      self.type + "\t" + self.id + "\t" + self.idbar + "\t" + self.idtd
    );

    if (s) {
      this.clear();
    }
    return s;
  },

  /* @static */
  startCallback: function(id)
  {
    return true;
  },

  /* @static */
  loadingCallback: function(url, id)
  {
     var ids = id.split("\t");
     SyL_AjaxRequest.sendAsyncPost(
       url,
       ids[1],
       function(result)
       {
         if (result.progress) {
           if (ids[0] == 'smooth') {
             document.getElementById(ids[2]).style.width = result.progress + "%";
           } else {
             var idss  = ids[2].split(",");
             var num   = idss.length;
             var point = 100 / num;
             for (var i=0; i<num; i++) {
               if (result.progress >= Math.ceil(point * (i+1))) {
                 document.getElementById(idss[i]).style.backgroundColor  = "#0B2467";
               } else {
                 break;
               }
             }
           }
           if (ids[3]) {
             document.getElementById(ids[3]).innerHTML = result.progress + "%";
           }
         }
       },
       null,
       {'finish': '0'}
     );
  },

  /* @static */
  completeCallback: function(url, body, id)
  {
     var ids = id.split("\t");
     SyL_AjaxRequest.sendAsyncPost(
       url,
       ids[1],
       function(result)
       {
         if (result.progress < 100) {
           alert("Request Timeout ? (" + result.progress + "% end)");
         }
         if (result.progress) {
           if (ids[0] == 'smooth') {
             document.getElementById(ids[2]).style.width = result.progress + "%";
           } else {
             var idss  = ids[2].split(",");
             var num   = idss.length;
             var point = 100 / num;
             for (var i=0; i<num; i++) {
               if (result.progress >= Math.ceil(point * (i+1))) {
                 document.getElementById(idss[i]).style.backgroundColor = "#0B2467";
               } else {
                 break;
               }
             }
           }
           if (ids[3]) {
             document.getElementById(ids[3]).innerHTML = result.progress + "%";
           }
         }
       },
       null,
       {'finish': '1'}
     );
  }

}

